@@ -807,6 +807,42 @@ pango_fc_patterns_get_pattern (PangoFcPatterns *pats)
   return pats->pattern;
 }
 
+static gboolean
+pango_fc_is_supported_font_format (const char *fontformat)
+{
+  /* harfbuzz supports only SFNT fonts. */
+  /* FIXME: "CFF" is used for both CFF in OpenType and bare CFF files, but
+   * HarfBuzz does not support the later and FontConfig does not seem
+   * to have a way to tell them apart.
+   */
+  if (g_ascii_strcasecmp (fontformat, "TrueType") == 0 ||
+      g_ascii_strcasecmp (fontformat, "CFF") == 0)
+    return TRUE;
+  return FALSE;
+}
+
+static FcFontSet *
+filter_fontset_by_format (FcFontSet *fontset)
+{
+  FcFontSet *result;
+  int i;
+
+  result = FcFontSetCreate ();
+
+  for (i = 0; i < fontset->nfont; i++)
+    {
+      FcResult res;
+      const char *s;
+
+      res = FcPatternGetString (fontset->fonts[i], FC_FONTFORMAT, 0, (FcChar8 **)(void*)&s);
+      g_assert (res == FcResultMatch);
+      if (pango_fc_is_supported_font_format (s))
+        FcFontSetAdd (result, FcPatternDuplicate (fontset->fonts[i]));
+    }
+
+  return result;
+}
+
 static FcPattern *
 pango_fc_patterns_get_font_pattern (PangoFcPatterns *pats, int i, gboolean *prepare)
 {
@@ -827,7 +863,17 @@ pango_fc_patterns_get_font_pattern (PangoFcPatterns *pats, int i, gboolean *prep
       if (!pats->fontset)
         {
 	  FcResult result;
-	  pats->fontset = FcFontSort (pats->fontmap->priv->config, pats->pattern, FcTrue, NULL, &result);
+          FcFontSet *fontset;
+          FcFontSet *filtered;
+
+	  fontset = FcFontSort (pats->fontmap->priv->config, pats->pattern, FcFalse, NULL, &result);
+          filtered = filter_fontset_by_format (fontset);
+          FcFontSetDestroy (fontset);
+
+          pats->fontset = FcFontSetSort (pats->fontmap->priv->config, &filtered, 1, pats->pattern, FcTrue, NULL, &result);
+
+          FcFontSetDestroy (filtered);
+
 	  if (pats->match)
 	    {
 	      FcPatternDestroy (pats->match);
@@ -896,24 +942,18 @@ pango_fc_fontset_get_key (PangoFcFontset *fontset)
 }
 
 static PangoFont *
-pango_fc_fontset_load_next_font (PangoFcFontset *fontset,
-                                 gboolean       *has_more)
+pango_fc_fontset_load_next_font (PangoFcFontset *fontset)
 {
   FcPattern *pattern, *font_pattern;
   PangoFont *font;
   gboolean prepare;
-  FcResult res;
-  const char *s;
 
   pattern = pango_fc_patterns_get_pattern (fontset->patterns);
   font_pattern = pango_fc_patterns_get_font_pattern (fontset->patterns,
 						     fontset->patterns_i++,
 						     &prepare);
   if (G_UNLIKELY (!font_pattern))
-    {
-      *has_more = FALSE;
-      return NULL;
-    }
+    return NULL;
 
   if (prepare)
     {
@@ -923,35 +963,26 @@ pango_fc_fontset_load_next_font (PangoFcFontset *fontset,
 	return NULL;
     }
 
-  res = FcPatternGetString (font_pattern, FC_FONTFORMAT, 0, (FcChar8 **)(void*)&s);
-  g_assert (res == FcResultMatch);
-  /* harfbuzz does not support these */
-  if (strcmp (s, "Type 1") == 0 || strcmp (s, "PCF") == 0)
-    font = NULL;
-  else
-    font = pango_fc_font_map_new_font (fontset->key->fontmap,
-                                       fontset->key,
-                                       font_pattern);
+  font = pango_fc_font_map_new_font (fontset->key->fontmap,
+                                     fontset->key,
+                                     font_pattern);
 
   if (prepare)
     FcPatternDestroy (font_pattern);
 
-  *has_more = TRUE;
-
   return font;
 }
 
 static PangoFont *
 pango_fc_fontset_get_font_at (PangoFcFontset *fontset,
-			      unsigned int    i,
-                              gboolean       *has_more)
+			      unsigned int    i)
 {
   while (i >= fontset->fonts->len)
     {
-      PangoFont *font = pango_fc_fontset_load_next_font (fontset, has_more);
+      PangoFont *font = pango_fc_fontset_load_next_font (fontset);
       g_ptr_array_add (fontset->fonts, font);
       g_ptr_array_add (fontset->coverages, NULL);
-      if (!*has_more)
+      if (!font)
         return NULL;
     }
 
@@ -1028,20 +1059,11 @@ pango_fc_fontset_get_font (PangoFontset  *fontset,
   PangoCoverage *coverage;
   int result = -1;
   unsigned int i;
-  gboolean has_more;
 
-  for (i = 0; TRUE; i++)
+  for (i = 0;
+       pango_fc_fontset_get_font_at (fcfontset, i);
+       i++)
     {
-      font = pango_fc_fontset_get_font_at (fcfontset, i, &has_more);
-
-      if (font == NULL)
-        {
-          if (has_more)
-            continue;
-          else
-            break;
-        }
-
       coverage = g_ptr_array_index (fcfontset->coverages, i);
 
       if (coverage == NULL)
@@ -1077,21 +1099,12 @@ pango_fc_fontset_foreach (PangoFontset           *fontset,
 {
   PangoFcFontset *fcfontset = PANGO_FC_FONTSET (fontset);
   PangoFont *font;
-  gboolean has_more;
   unsigned int i;
 
-  for (i = 0; TRUE; i++)
+  for (i = 0;
+       (font = pango_fc_fontset_get_font_at (fcfontset, i));
+       i++)
     {
-      font = pango_fc_fontset_get_font_at (fcfontset, i, &has_more);
-
-      if (font == NULL)
-        {
-          if (has_more)
-            continue;
-          else
-            break;
-        }
-
       if ((*func) (fontset, font, data))
 	return;
     }
@@ -1394,8 +1407,8 @@ pango_fc_font_map_list_families (PangoFontMap      *fontmap,
 
 	  res = FcPatternGetString (fontset->fonts[i], FC_FONTFORMAT, 0, (FcChar8 **)(void*)&s);
 	  g_assert (res == FcResultMatch);
-          if (strcmp (s, "Type 1") == 0 || strcmp (s, "PCF") == 0)
-            continue; /* harfbuzz does not support these */
+          if (!pango_fc_is_supported_font_format (s))
+            continue;
 
 	  res = FcPatternGetString (fontset->fonts[i], FC_FAMILY, 0, (FcChar8 **)(void*)&s);
 	  g_assert (res == FcResultMatch);
